/*
 * This file is a component of thundr, a software library from 3wks.
 * Read more: http://www.3wks.com.au/thundr
 * Copyright (C) 2013 3wks, <thundr@3wks.com.au>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.threewks.thundr.action.method.bind.request;

import static com.atomicleopard.expressive.Expressive.list;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jodd.util.StringPool;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.text.WordUtils;

import com.atomicleopard.expressive.Expressive;
import com.threewks.thundr.action.method.bind.ActionMethodBinder;
import com.threewks.thundr.action.method.bind.http.HttpBinder;
import com.threewks.thundr.introspection.ParameterDescription;

public class RequestHeaderBinder implements ActionMethodBinder {
	private HttpBinder delegate;

	public RequestHeaderBinder(HttpBinder delegate) {
		this.delegate = delegate;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void bindAll(Map<ParameterDescription, Object> bindings, HttpServletRequest req, HttpServletResponse resp, Map<String, String> pathVariables) {
		for (ParameterDescription parameterDescription : bindings.keySet()) {
			if (bindings.get(parameterDescription) == null) {
				Enumeration<String> headerNames = req.getHeaderNames();
				if (headerNames != null) {
					Map<String, String[]> headerValues = new HashMap<String, String[]>();
					for (String header : Expressive.<String> iterable(headerNames)) {
						headerValues.put(normaliseHeaderName(header), headerValues(req.getHeaders(header)));
					}

					delegate.bind(bindings, req, resp, headerValues);
				}
			}
		}
	}

	String normaliseHeaderName(String header) {
		String capitalised = WordUtils.capitalizeFully(header, '-');
		String withoutDashes = capitalised.replaceAll(StringPool.DASH, StringPool.EMPTY);
		return StringUtils.uncapitalize(withoutDashes);
	}

	private String[] headerValues(Enumeration<String> headers) {
		return list(Expressive.iterable(headers)).toArray(new String[0]);
	}
}
